//==============================================================================
// PROJECT      = H_Line Demonstration Board EB-V850ES/HG2-EE (Count Down Timer)
// MODULE       = timer.c
// SHORT DESC.  = -
// DEVICE       = V850ES/HG2 (uPD70F3707)
// VERSION      = 1.0
// DATE         = 08.11.2005
// LAST CHANGE  = -
// ===========================================================================
// Description:  Count-down timer
//               Key SW2 is used to set the timer to 1 (= 1 key stroke) to 8
//               (= 8 key strokes) minutes. By pressing key SW1, the count-down
//               process is started. After it has finished the 7seg LED is flashing
//               and an external piezo speaker beeps. It goes back to the stand
//               by state after 3 minutes of blinking or a key press by the
//               customer. During count down time the dot segment of 7seg LED
//               is flashing.
//               A key stroke during the countdown takes the unit to the time
//               input state.
// ===========================================================================
// By:           NEC Electronics (Europe) GmbH
//               Arcadiastrasse 10
//               D-40472 Duesseldorf
//
// =============================================================================


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "defines.h"

#include <intrinsics.h>
#include "io70f3707.h"

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
       void   vFlashLED      (unsigned char);  // flash LED's
       void   vWait25        (unsigned char);  // n *  25ms delay
extern void   vHardwareInit  (void);           // initialize hardware
extern void   vSoftwareInit  (void);           // initialize variables

       void   _STOP          (void);           // switch to STOP mode

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
int            uiCountdownTime;                // countdown Time [sec]
unsigned char  ucState=INPUT;                  // state

volatile unsigned char  bTimer2Flag;           // status Flag Timer2
volatile unsigned char  bKeySW1Flag;           // status Flag KeySW1
volatile unsigned char  bKeySW2Flag;           // status Flag KeySW2

// ============================================================================
//  Module name: LED_segout
//
//  Description:
//               Generates output pattern for 7 seg. LED
//
// ============================================================================
unsigned char LED_segout(unsigned char binary)
{
    const unsigned char tab[16]={0xC0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x83,0xc6,0xa1,0x86,0x8e};
    return tab[binary];
}

// ============================================================================
//  Module name: vFlashLED
//
//  Description:
//               This module flashes all LEDs n times.
//
//  Operation:
//               -
// ============================================================================

void vFlashLED(unsigned char Number)
{
    while(Number>0) {
       P9L = 0x00;                   // all LEDs on
       vWait25(40);                  // delay of 1 s
       P9L = 0xff;                   // all LEDs off
       vWait25(40);                  // delay of 1 s
       Number--;
    }
    return;
}
// ============================================================================
//  Module name: vWait25
//
//  Description:
//               This module delays the program for n * 25ms.
//
//  Operation:
//               Reset the timer status flag, start the timer and wait
//               for the given number of seconds. Stop the Timer.
// ============================================================================

void vWait25(unsigned char Number)
{
  bTimer2Flag = FALSE;                 // reset status flag Timer2
  STARTTIMER;                         // start Timer2
  while(Number > 0) {
        while(bTimer2Flag == FALSE){   // wait for Timer Interrupt
          __NOP();
          __NOP();
        }
        TRIGGERWD;                     // trigger watchdog
        bTimer2Flag = FALSE;           // reset status flag Timer2
        Number--;
  }
  STARTTIMER;                         // stop Timer2
  return;
}

// ============================================================================
//  main function
// ============================================================================

unsigned char counter;

void main (void)
{
  unsigned int uiFlashCount;
  TRIGGERWD;                           // trigger watchdog
  __DI();                              // Disable all interrupts
  vHardwareInit();                     // Peripheral settings
  vSoftwareInit();                     // Variable Initialization
  __EI();                              // Enable all interrupts
  vFlashLED(2);                        // Signal Program start

  while(1)
  {
      switch (ucState) {
          case INPUT:
             TRIGGERWD;                                         // trigger watchdog
             if(bKeySW2Flag == TRUE){
                bKeySW2Flag = FALSE;
                if(uiCountdownTime <= 420){
                  uiCountdownTime += 60;
                  P9L = LED_segout(uiCountdownTime/60);
                }
             }
             if(bKeySW1Flag == TRUE){
                bKeySW1Flag = FALSE;
                if(uiCountdownTime > 0){
                   ucState = TIMER;
                }
             }
             break;
          case TIMER :
             STARTTIMER;                                         // start Timer2
             while(uiCountdownTime>0x00){
                vWait25(40);
                uiCountdownTime--;

                if((uiCountdownTime%60) == 0){
                  P9L = LED_segout(uiCountdownTime/60);
                }
                if((uiCountdownTime%2) == 0){                    // every second
                      P9L_bit.no7 = 0x01;                        // switch off DOT LED
                }
                else{                                            // or
                      P9L_bit.no7 = 0x00;                        // switch on DOT LED
                }
                if((bKeySW1Flag == TRUE) || (bKeySW2Flag == TRUE)){  // stop count-down
                  bKeySW1Flag       = FALSE;
                  bKeySW2Flag       = FALSE;
                  uiCountdownTime = -1;
                  P9L             = 0xff;
                }
             }
             STOPTIMER;                                         // stop Timer2
             if(uiCountdownTime < 0){
                 ucState=INPUT;
             }
             else{
                 ucState=TIMEREND;
             }
             uiCountdownTime = 0;
             break;

          case STANDBY :
             _STOP();
             bKeySW1Flag = FALSE;          // clear Key Flags after wake up from STOP mode
             bKeySW2Flag = FALSE;
             ucState   = INPUT;            // switch to input mode
             TRIGGERWD;                    // trigger watchdog
             vFlashLED(2);                 // Signal Program start
             break;

          case TIMEREND :
             uiFlashCount = 0;
             while(uiFlashCount <= 90){    // 3 min Flash time (90 x 2 secs)
                vFlashLED(1);              // 2 sec
                uiFlashCount++;
                if((bKeySW1Flag == TRUE) || (bKeySW2Flag == TRUE)){  // switch to stand-by
                  bKeySW1Flag    = FALSE;
                  bKeySW2Flag    = FALSE;
                  uiFlashCount = 91;
                }
             }
             ucState  = STANDBY;
             break;
          default:
             ucState = STANDBY;
             break;
      }
  }
}

// ============================================================================
//  Module name: _STOP()
//
//  Description:
//               This module switches to stop mode
//
//  Operation:
//               -
// ============================================================================

void _STOP (void)
{
    PSMR= 0x01;                             // Select STOP mode
    PRCMD= 0;                               // Switch to STOP mode
    PSC= 0x02;
    __asm("nop");
    __asm("nop");
    __asm("nop");
    __asm("nop");
    __asm("nop");
}

