//==============================================================================
// PROJECT      = H_Line Demonstration Board EB-V850ES/HG2-EE (Melody Maker)
// MODULE       = melody.c
// SHORT DESC.  = -
// DEVICE       = V850ES/HG2 (uPD70F3707)
// VERSION      = 1.0
// DATE         = 08.11.2005
// LAST CHANGE  = -
// ===========================================================================
// Description:  Melody maker
//               An external piezo speaker (provided by the customer) plays an
//               predefined melody. The definition of the melody is done in the
//               program code.
//               Each push of KeySW1 selects the next melody, a push of KeySW2
//               selects the previous melody.
// ===========================================================================
// By:           NEC Electronics (Europe) GmbH
//               Arcadiastrasse 10
//               D-40472 Duesseldorf
//
// =============================================================================

//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "defines.h"

#include <intrinsics.h>
#include "io70f3707.h"


//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
       void   vFlashLED     (unsigned char);               // Flash LED's
       void   vWait275      (unsigned char);                // n * 55ms delay
       void   vPlayMelody   (const unsigned char *);       // Plays selected melody
extern void   vHardwareInit (void);                        // Initialize hardware
extern void   vSoftwareInit (void);                        // Initialize variables

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
volatile unsigned char bTimer2Flag;    // Status Flag Timer2
volatile unsigned char bKeySW1Flag;    // Status Flag KeySW1
volatile unsigned char bKeySW2Flag;    // Status Flag KeySW2
          unsigned char ucLEDs;        // LED output value

//-----------------------------------------------------------------------------
// Global constants
//-----------------------------------------------------------------------------

// Melody Definition:
// Each two elements represent one tone: first element  => on  - time
//                                       second element => off - Time
// Definition of melody 8
const unsigned char ucData8[]={EI,EI,FU,EI,EI,EI,FU,EI,EI,EI,FU,EI,EI,EI,FU,EI,END};

// Definition of melody 7
const unsigned char ucData7[]={EI,SI,FU,SI,FU,SI,FU,SI,EI,SI,FU,SI,FU,SI,FU,SI,END};

// Definition of melody 6
const unsigned char ucData6[]={EI,FO,EI,FO,EI,FU,HA,FO,HA,FO,HA,FU,EI,FO,EI,FO,EI,FU,END};

// Definition of melody 5
const unsigned char ucData5[]={EI,SI,EI,SI,HA,HA,EI,SI,EI,SI,HA,HA,END};

// Definition of melody 4
const unsigned char ucData4[]={EI,FO,FO,FO,HA,FO,FU,FO,HA,FO,FO,FO,EI,FO,END};

// Definition of melody 3
const unsigned char ucData3[]={FU,EI,HA,EI,FO,EI,EI,EI,FO,EI,HA,EI,FU,EI,END};

// Definition of melody 2
const unsigned char ucData2[]={FU,SI,HA,SI,FU,SI,HA,SI,FU,SI,HA,SI,FU,SI,END};

// Definition of melody 1
const unsigned char ucData1[]={EI,SI,EI,SI,EI,SI,FU,SI,EI,SI,EI,SI,EI,SI,END};


// ============================================================================
//  Module name: LED_segout
//
//  Description:
//               Generates output pattern for 7 seg. LED
//
// ============================================================================
unsigned char LED_segout(unsigned char binary)
{
    const unsigned char tab[16]={0xC0,0xf9,0xa4,0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x83,0xc6,0xa1,0x86,0x8e};
    return tab[binary];
}

// ============================================================================
//  Module name: vPlayMelody
//
//  Description:
//               This module plays the selected melody
//
//  Operation:
//               -
// ============================================================================

void vPlayMelody(const unsigned char *pData)
{
    while ((*pData != END)&&(*(pData+1)!=END)){
         PIEZO=PIEZO_ON;
         vWait275(*pData);             // Wait for n * 27,5 ms (ON-Time)
         pData++;
         PIEZO=PIEZO_OFF;
         vWait275(*pData);             // Wait for n * 27,5 ms (OFF-Time)
         pData++;
    }
    vWait275(120);                     // 3.3 s delay at melody end
    return;
}



// ============================================================================
//  Module name: vFlashLED
//
//  Description:
//               This module flashes all LEDs n times.
//
//  Operation:
//               -
// ============================================================================

void vFlashLED(unsigned char Number)
{
    while(Number>0) {
       ucLEDs=0x00;                    // all LEDs on
       vWait275(20);                   // Delay of 550 ms
       ucLEDs=0xFF;                    // all LEDs off
       vWait275(20);                   // Delay of 550 ms
       Number--;
    }
    return;
}


// ============================================================================
//  Module name: vWait275
//
//  Description:
//               This module delays the program for n * 27,5 ms.
//
//  Operation:
//               -
// ============================================================================

void vWait275(unsigned char Number)
{
    bTimer2Flag = FALSE;               // Reset Timer2 flag
    STARTTIMER2;                       // Start Timer2
    while(Number>0) {
         while(bTimer2Flag == FALSE){  // wait for Timer Interrupt
          __NOP();
          __NOP();
        }
        TRIGGERWD;                     // trigger watchdog
        bTimer2Flag = FALSE;           // Reset status flag Timer2
        Number--;
    }
    STOPTIMER2;                        // Stop Timer2
    return;
}



// ============================================================================
//  main function
// ============================================================================

void main( void )
{
    unsigned char i;
    const unsigned char * pData;
    __DI();                            // Disable all interrupts
    vHardwareInit();                   // Peripheral settings
    vSoftwareInit();                   // Variable Initialization
    __EI();                            // Enable all interrupts
    STARTTIMER1;                       // Start Timer50
    vFlashLED(2);                      // Signal Program start
    while(TRUE)
    {
       if(bKeySW1Flag==1){               // KeySW1 pressed ?
          bKeySW1Flag=0;                 // Reset status flag KeySW1
          if(i<7)
            i++;
          else
            i=0;
       }
       if(bKeySW2Flag==1){               // KeySW2 pressed ?
          bKeySW2Flag=0;                 // Reset status flag KeySW2
          if(i>0)
            i--;
          else
            i=7;
       }
       ucLEDs = LED_segout(i);         // Indicate the selected melody
       switch (i){                     // Get the selected melody data
          case 0:
                 pData=&ucData1[0];
                 break;
          case 1:
                 pData=&ucData2[0];
                 break;
          case 2:
                 pData=&ucData3[0];
                 break;
          case 3:
                 pData=&ucData4[0];
                 break;
          case 4:
                 pData=&ucData5[0];
                 break;
          case 5:
                 pData=&ucData6[0];
                 break;
          case 6:
                 pData=&ucData7[0];
                 break;
          case 7:
                 pData=&ucData8[0];
                 break;
          default:
                 pData=&ucData1[0];
                 break;
       }
       vPlayMelody(pData);             // Play the selected melody
    }
}
